package kr.co.vividnext.sodalive.live

import android.annotation.SuppressLint
import android.content.Intent
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.webkit.URLUtil
import android.widget.LinearLayout
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.zhpan.bannerview.BaseBannerAdapter
import com.zhpan.indicator.enums.IndicatorSlideMode
import com.zhpan.indicator.enums.IndicatorStyle
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.base.BaseFragment
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.common.SharedPreferenceManager
import kr.co.vividnext.sodalive.databinding.FragmentLiveBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.live.event_banner.EventBannerAdapter
import kr.co.vividnext.sodalive.live.now.LiveNowAdapter
import kr.co.vividnext.sodalive.live.recommend.RecommendLiveAdapter
import kr.co.vividnext.sodalive.live.recommend_channel.LiveRecommendChannelAdapter
import kr.co.vividnext.sodalive.live.reservation.LiveReservationAdapter
import kr.co.vividnext.sodalive.settings.notification.MemberRole
import org.koin.android.ext.android.inject
import kotlin.math.roundToInt

class LiveFragment : BaseFragment<FragmentLiveBinding>(FragmentLiveBinding::inflate) {
    private val viewModel: LiveViewModel by inject()

    private lateinit var liveNowAdapter: LiveNowAdapter
    private lateinit var liveReservationAdapter: LiveReservationAdapter
    private lateinit var liveRecommendChannelAdapter: LiveRecommendChannelAdapter

    private lateinit var loadingDialog: LoadingDialog

    private var message = ""
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setupView()
        setupRecommendLive()
        setupRecommendChannel()
        setupLiveNow()
        setupLiveReservation()
        setupEvent()

        message = "라이브를 불러오고 있습니다."
        viewModel.getSummary()
    }

    private fun setupView() {
        loadingDialog = LoadingDialog(requireActivity(), layoutInflater)

        viewModel.toastLiveData.observe(viewLifecycleOwner) {
            it?.let { Toast.makeText(requireActivity(), it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(viewLifecycleOwner) {
            if (it) {
                loadingDialog.show(screenWidth, message)
            } else {
                loadingDialog.dismiss()
            }
        }

        binding.ivMakeRoom.visibility =
            if (SharedPreferenceManager.role == MemberRole.CREATOR.name) {
                View.VISIBLE
            } else {
                View.GONE
            }
        binding.ivMakeRoom.setOnClickListener {}

        binding.swipeRefreshLayout.setOnRefreshListener { refreshSummary() }

        val ivHowToUseLp = binding.ivHowToUse.layoutParams as LinearLayout.LayoutParams
        ivHowToUseLp.width = screenWidth
        ivHowToUseLp.height = (200 * screenWidth) / 1080
        binding.ivHowToUse.layoutParams = ivHowToUseLp
        binding.ivHowToUse.setOnClickListener {
            val url = "https://blog.naver.com/yozmlive"
            if (URLUtil.isValidUrl(url)) {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
            }
        }
    }

    private fun refreshSummary() {
        liveNowAdapter.clear()
        liveRecommendChannelAdapter.clear()
        liveReservationAdapter.clear()

        message = "라이브를 불러오고 있습니다."
        viewModel.getSummary()

        binding.swipeRefreshLayout.isRefreshing = false
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupRecommendLive() {
        val layoutParams = binding
            .layoutRecommendLive
            .pager
            .layoutParams as LinearLayout.LayoutParams

        val pagerWidth = screenWidth.toDouble() - 26.7f.dpToPx()
        val pagerHeight = (pagerWidth * 0.53).roundToInt()
        layoutParams.width = pagerWidth.roundToInt()
        layoutParams.height = pagerHeight

        binding
            .layoutRecommendLive
            .pager
            .layoutParams = layoutParams

        binding.layoutRecommendLive.pager.apply {
            adapter = RecommendLiveAdapter(pagerWidth.roundToInt(), pagerHeight) {
            } as BaseBannerAdapter<Any>

            setLifecycleRegistry(lifecycle)
            setScrollDuration(1000)
            setInterval(4 * 1000)
        }.create()

        binding
            .layoutRecommendLive
            .pager
            .setIndicatorView(binding.layoutRecommendLive.indicator2)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_9970ff)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.recommendLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.layoutRecommendLive.root.visibility = View.VISIBLE
                binding.layoutRecommendLive.pager.refreshData(it)
            } else {
                binding.layoutRecommendLive.root.visibility = View.GONE
            }
        }
    }

    private fun setupRecommendChannel() {
        liveRecommendChannelAdapter = LiveRecommendChannelAdapter(
            onClick = {},
            onClickMore = {}
        )

        binding.layoutRecommendChannel.rvRecommendChannel.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        binding
            .layoutRecommendChannel
            .rvRecommendChannel
            .addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)

                    when (parent.getChildAdapterPosition(view)) {
                        0 -> {
                            outRect.left = 0.dpToPx().toInt()
                            outRect.right = 8.3f.dpToPx().toInt()
                        }

                        liveRecommendChannelAdapter.itemCount - 1 -> {
                            outRect.left = 8.3f.dpToPx().toInt()
                            outRect.right = 0.dpToPx().toInt()
                        }

                        else -> {
                            outRect.left = 8.3f.dpToPx().toInt()
                            outRect.right = 8.3f.dpToPx().toInt()
                        }
                    }
                }
            })

        binding.layoutRecommendChannel.rvRecommendChannel.adapter = liveRecommendChannelAdapter
        binding.layoutRecommendChannel.ivSwitch.setOnClickListener {
            viewModel.toggleIsFollowedCreatorLive()
        }

        viewModel.recommendChannelLiveData.observe(viewLifecycleOwner) {
            binding.layoutRecommendChannel.root.visibility = View.VISIBLE
            liveRecommendChannelAdapter.addItems(it)
            binding.layoutRecommendChannel.rvRecommendChannel.requestLayout()
            binding.layoutRecommendChannel.root.requestLayout()
        }

        viewModel.isFollowedCreatorLive.observe(viewLifecycleOwner) {
            liveRecommendChannelAdapter.isFollowedCreatorLive = it
            liveRecommendChannelAdapter.clear()

            if (it) {
                binding.layoutRecommendChannel.ivSwitch.setImageResource(R.drawable.btn_toggle_on_big)
                binding.layoutRecommendChannel.llTitle2.visibility = View.VISIBLE
                binding.layoutRecommendChannel.llTitle1.visibility = View.GONE
            } else {
                binding.layoutRecommendChannel.ivSwitch.setImageResource(R.drawable.btn_toggle_off_big)
                binding.layoutRecommendChannel.llTitle1.visibility = View.VISIBLE
                binding.layoutRecommendChannel.llTitle2.visibility = View.GONE
            }
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupLiveNow() {
        binding
            .layoutLiveNow
            .tvAllView
            .setOnClickListener {}

        val recyclerView = binding
            .layoutLiveNow
            .rvSudaNow

        liveNowAdapter = LiveNowAdapter {}

        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.HORIZONTAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.left = 13.3f.dpToPx().toInt()
                        outRect.right = 5.dpToPx().toInt()
                    }

                    liveNowAdapter.itemCount - 1 -> {
                        outRect.left = 5.dpToPx().toInt()
                        outRect.right = 13.3f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.left = 5.dpToPx().toInt()
                        outRect.right = 5.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = liveNowAdapter

        viewModel.liveNowLiveData.observe(viewLifecycleOwner) {
            if (liveNowAdapter.items.isEmpty() && it.isEmpty()) {
                recyclerView.visibility = View.GONE
                binding.layoutLiveNow.tvAllView.visibility = View.GONE
                binding.layoutLiveNow.llNoItems.visibility = View.VISIBLE
                binding.layoutLiveNow.tvMakeRoom.setOnClickListener {}

                recyclerView.requestLayout()
                binding.layoutLiveNow.llNoItems.requestLayout()
            } else {
                binding.layoutLiveNow.tvAllView.visibility = View.VISIBLE
                binding.layoutLiveNow.llNoItems.visibility = View.GONE
                liveNowAdapter.items.addAll(it)
                liveNowAdapter.notifyDataSetChanged()
                recyclerView.visibility = View.VISIBLE
                recyclerView.requestLayout()
            }

            binding.layoutLiveNow.root.requestLayout()
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupLiveReservation() {
        val recyclerView = binding
            .layoutLiveReservation
            .rvSudaReservation

        liveReservationAdapter = LiveReservationAdapter(isMain = true) {}

        recyclerView.layoutManager = LinearLayoutManager(
            requireContext(),
            LinearLayoutManager.VERTICAL,
            false
        )

        recyclerView.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()

                when (parent.getChildAdapterPosition(view)) {
                    0 -> {
                        outRect.top = 0f.dpToPx().toInt()
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }

                    liveReservationAdapter.itemCount - 1 -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 13.3f.dpToPx().toInt()
                    }

                    else -> {
                        outRect.top = 6.7f.dpToPx().toInt()
                        outRect.bottom = 6.7f.dpToPx().toInt()
                    }
                }
            }
        })

        recyclerView.adapter = liveReservationAdapter

        viewModel.liveReservationLiveData.observe(viewLifecycleOwner) {
            if (liveReservationAdapter.items.isEmpty() && it.isEmpty()) {
                recyclerView.visibility = View.GONE
                binding.layoutLiveReservation.tvAllView.visibility = View.GONE
                binding.layoutLiveReservation.llNoItems.visibility = View.VISIBLE
                binding.layoutLiveReservation.tvMakeRoom.setOnClickListener {}

                recyclerView.requestLayout()
                binding.layoutLiveReservation.llNoItems.requestLayout()
            } else {
                binding.layoutLiveReservation.tvAllView.visibility = View.VISIBLE
                binding.layoutLiveReservation.llNoItems.visibility = View.GONE
                liveReservationAdapter.items.addAll(it)
                liveReservationAdapter.notifyDataSetChanged()

                recyclerView.visibility = View.VISIBLE
                recyclerView.requestLayout()
            }

            binding.layoutLiveReservation.root.requestLayout()
        }

        binding.layoutLiveReservation.tvAllView.setOnClickListener {}
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun setupEvent() {
        val imageSliderLp = binding.eventBannerSlider.layoutParams
        imageSliderLp.width = screenWidth
        imageSliderLp.height = (screenWidth * 300) / 1000
        binding.eventBannerSlider.layoutParams = imageSliderLp

        binding.eventBannerSlider.apply {
            adapter = EventBannerAdapter(requireContext()) {} as BaseBannerAdapter<Any>
            setLifecycleRegistry(lifecycle)
            setScrollDuration(800)
        }.create()

        binding.eventBannerSlider
            .setIndicatorView(binding.indicator)
            .setIndicatorStyle(IndicatorStyle.ROUND_RECT)
            .setIndicatorSlideMode(IndicatorSlideMode.SMOOTH)
            .setIndicatorVisibility(View.GONE)
            .setIndicatorSliderColor(
                ContextCompat.getColor(requireContext(), R.color.color_909090),
                ContextCompat.getColor(requireContext(), R.color.color_9970ff)
            )
            .setIndicatorSliderWidth(4f.dpToPx().toInt(), 10f.dpToPx().toInt())
            .setIndicatorHeight(4f.dpToPx().toInt())

        viewModel.eventLiveData.observe(viewLifecycleOwner) {
            if (it.isNotEmpty()) {
                binding.eventBannerSlider.visibility = View.VISIBLE
                binding.eventBannerSlider.refreshData(it)
            } else {
                binding.eventBannerSlider.visibility = View.GONE
            }
        }
    }
}
